// $Id: MusicStatus.java,v 1.10 2003/06/09 16:20:52 xi Exp $

/******************************************************************************
*    MusicControl - An advanced music player
*    Copyright (C) 2002,2003  Christian Laursen <xi@borderworlds.dk>
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/

package dk.borderworlds;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.util.Vector;

public class MusicStatus implements ActionListener,Runnable {
	private JPanel panel;
	private MusicQueue queue1, queue2;
	private JButton play, stop, skip;
	private JLabel statusLabel, songLabel, posLabel, lengthLabel;
	private MusicPlayer[] players;
	private MusicPlayer player;
	private Thread updater;
	private boolean skipping;

	public MusicStatus (MusicQueue q1, MusicQueue q2, MusicPlayer[] players) {
		queue1 = q1;
		queue2 = q2;
		this.players = players;
		player = null;
		skipping = false;

		panel = new JPanel(new BorderLayout());
		panel.add(new JLabel("Play status"), BorderLayout.NORTH);

		JPanel stPanel = new JPanel(new GridLayout(4,2));

		stPanel.add(new JLabel("Status:"));
		statusLabel = new JLabel("Stopped");
		stPanel.add(statusLabel);

		stPanel.add(new JLabel("Song:"));
		songLabel = new JLabel("(none)");
		stPanel.add(songLabel);

		stPanel.add(new JLabel("Position:"));
		posLabel = new JLabel("N/A");
		stPanel.add(posLabel);

		stPanel.add(new JLabel("Length:"));
		lengthLabel = new JLabel("N/A");
		stPanel.add(lengthLabel);

		panel.add(stPanel, BorderLayout.CENTER);

		JPanel actionPanel = new JPanel(new GridLayout(1,3));
		play = new JButton("Play");
		play.addActionListener(this);
		stop = new JButton("Stop");
		stop.addActionListener(this);
		skip = new JButton("Skip");
		skip.addActionListener(this);
		actionPanel.add(play);
		actionPanel.add(stop);
		actionPanel.add(skip);
		panel.add(actionPanel, BorderLayout.SOUTH);
		updater = new Thread(this);
		updater.start();
	}

	public Component getComponent () {
		return panel;
	}

	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();

		if (command.equals("Play") || command.equals("Skip")) {
			playNext();
		} else if (command.equals("Pause")) {
			player.pauseSong();
			play.setText("Resume");
		} else if (command.equals("Resume")) {
			player.resumeSong();
			play.setText("Pause");
		} else if (command.equals("Stop")) {
			if (player != null) {
				skipping = true;
				player.stopSong();
				player = null;
				play.setText("Play");
				skipping = false;
			}
		}
	}

	public void stopPlaying () {
		if (player != null) {
			skipping = true;
			player.stopSong();
			player = null;
		}
		skipping = false;
	}

	private void playNext () {
		Song song;
		int i;

		if (player != null) {
			skipping = true;
			player.stopSong();
		}
		song = queue1.getSong();
		if (song == null) {
			song = queue2.getSong();
		}
		if (song != null) {
			for (i = 0; i < players.length; ++i) {
				if (players[i].acceptsType(song.getType())) {
					player = players[i];
					i = players.length;
				}
			}
			if (player != null) {
				player.playSong(song);
				play.setText("Pause");
			}
		} else {
			player = null;
			play.setText("Play");
		}
		skipping = false;
	}

	public void run () {
		while (true) {
			//			System.err.println("Checking status");
			if (player != null) {
				String status = player.getStatus();
				statusLabel.setText(status);
				songLabel.setText(player.getTitle());
				posLabel.setText(player.getPosition());
				lengthLabel.setText(player.getLength());
				if (status.equals("Stopped") && !skipping) {
					player = null;
					playNext();
				}
			} else {
				statusLabel.setText("Stopped");
				songLabel.setText("(none)");
			}
			try {
				Thread.sleep(1000);
			} catch (Exception e) {
			}
		}
	}
}
